// this is now a dummy class for the XLM just to permit the GUI to run

#include <config.h>
#include "/usr/src/install/llnlReadout-3.4/Globals.h"
#include <vector>
#include <stdio.h>
#include <math.h>
#include <string.h>
using namespace std;
#include <iostream>
#include <fstream>
#include <ctype.h>
#include <stdlib.h>

//#include "param.h"

#define XLM_CONFIG_LENGTH 42000

#include "XLM.h"

const static int BUSRETRYTIME(5); // usec delay between bus access retry.
static struct usb_device* vmUsbDevice;
uint32_t ID;
static CVMUSBReadoutList VMEList;
static uint32_t XLM_Base;    // VME base address of XLM

// Constructors
/*!
XLM object constructor. Derives from CVMUSB Class and maps unsigned longs
onto the address space of the VME Module class.
*/

XLM::XLM (string name, unsigned int crate, unsigned int slot) :
  AVmeModule(name, crate, slot)
{
  vector<struct usb_device*> controllers = CVMUSB::enumerate();
  if (controllers.size() == 0) {
    cerr << "There appear to be no VMUSB controllers.\n";
    exit(-1);
  }
  printf("%d VMUSB devices found.\n",controllers.size());
  Globals::pUSBController = new CVMUSB(controllers[0]);
  ID = Globals::pUSBController->readFirmwareID();
  printf("FirmwareID is %x\n",ID);
  XLM_Base = slot << 27;
  printf("XLM_Base = %x\n",XLM_Base);
	fileName  = new char[120];
	//	srama     = Map(0x000000, 0x2A000);
	//	sramb     = Map(0x200000, 0x2A000);
	//	fpga      = Map(0x400000, 0x20);
}

/*!
Destructor.
*/
XLM::~XLM()
{
  //	delete [] fileName;
}

/*
XLM::XLM(const XLM& rhs) :
  CVMUSB(rhs)
{
}
*/

XLM& XLM::operator= (const XLM& aXLM)
{
  if (this != &aXLM) {

    //    AVmeModule::operator= (aXLM);
  }
  return *this;
}

int
XLM::operator==(const XLM& rhs) const
{
  return (operator==(rhs));
}

// Functions for class XLM
// FPGA functions

/*!
Configuration function.  This function opens the configuration file and writes each
byte to a space in the ASRAM A.  The boot source is also set to ASRAM A, and the 
FPGA is boot from this.  All Busses are realeased at the end.
Configuration net files are generated by the user.   

\param configfile - Character pointer to the configuration file.

CAUTION:  The only check done by this function is the length of the net file.  In no way
does it verify the functionality, usefulness, correctness, or safety of the
resulting configuration.
*/
void
XLM::Configure(char configfile[80])
{
	FILE *file;
	int i, j;
	unsigned long word, llow, lhigh, check;
	unsigned char blow, bhigh;
	unsigned char bytes[XLM_CONFIG_LENGTH];
	//string name = getName();
	//fileName = Param_getStrVal(name.c_str(), "configFileName");
	fileName = configfile;
	file = fopen(configfile, "r");
	if (!file) {
		cout << "Failed to open configuration file " << fileName << endl;
		exit(1);
	}
	int length = fread(bytes, 1, XLM_CONFIG_LENGTH, file);
	cout<<configfile<<" open with length "<<length<<" bytes."<<endl;
	if (!feof(file)) {
		cout << "Failed to reach EOF of configuration file " << fileName << endl;
		exit(1);
	}
	fclose(file);
	// Detect first 0xFF in file
	for (i=0; i<length; i++) {
		if (bytes[i] == 0xFF) break;
	}
	AccessBus(0x10001);
	// Translate and write configuration to SRAMA
	for (j=i; j<length; j++) {
		blow = bytes[j]&0xf;
		bhigh = (bytes[j]&0xf0)>>4;
		llow = ((blow&0x7)<<2) + ((blow&0x8)<<7);
		lhigh = ((bhigh&0x7)<<2) + ((bhigh&0x8)<<7);
		word = llow + (lhigh<<16);
		//	cout<<check<<" "<<srama[(j-i)]<<" "<<word<<endl;
		srama[(j-i)] = word;
		check = srama[(j-i)];
		//cout<<check<<" "<<srama[(j-i)]<<" "<<word<<endl;
		if (check != word) {
		  //			cout << "Invalid write to SRAMA configuring " 
		  //			     << getName() << " (slot " << getSlot() << " )" << endl;
			cout<<check<<" "<<srama[(j-i)]<<" "<<word<<endl;
			exit(1);
		}
	}
	// Set FPGA boot source to SRAMA
	SetFPGABoot(0x10000);
	BootFPGA();
	ReleaseBus();
}

/*!  
Checks the configuration of the loaded net file by accessing ASRAMA and invoking the CheckConfiguration
member function.  This is only used if the user has loaded a predefined word into the lowest address register
of ASRAM A.
*/
void
XLM::Initialize()
{
  AccessBus(0x10000);
  //	CheckConfiguration(0);
  ReleaseBus();
}

/*!
Opens the bus for VME access.  Before opening, checks to see if the busses are available.

\param busses - A logical OR of the busses to be accessed:
 - 0x00001: ASRAM A.
 - 0x00002: AXRAM B.
 - 0x10000: Bus X for FPGA access.
 - 0x20000: Bus B for DSP access.
*/
void
XLM::AccessBus(long busses)
{
	int attempts = 20;
	uint32_t check = 0;
	printf("AccessBus\n");
	check = busses;     // hack to make always work
	for(int i =0; i < attempts; i++) {
	  Globals::pUSBController->vmeWrite32(XLM_Base+0x800000,0x09,busses);
	  //		vme[0] = busses;
	  Globals::pUSBController->vmeRead32(XLM_Base+0x800000,0x09,&check);
	  //		check = vme[0]&0x00030003;
	  printf("AccessBus returns %x\n",check);
		attempts--;
		if(check == busses) {
		  break;
		}
		else {
		  usleep(BUSRETRYTIME);
		}
		
	} 
	if (check != busses) {
	  cout << "Failed to grab busses in " << getName() << " (slot " << getSlot() << " )" << endl;
	  //		exit(0);
	}
}

/*!
Releases busses from VME Access.  
*/
void
XLM::ReleaseBus()
{
	printf("ReleaseBus\n");
  //	vme[0] = 0;
}

/*!
Accesses the FPGA and writes to the reset register, forcing a boot from ASRAMA.  
Pauses for 0.5 seconds to allow the FPGA to read the boot source.
*/
void
XLM::BootFPGA()
{
  //	vme[1] = 1;
	usleep(10);
	//	vme[1] = 0;
	cout << "Booting FPGA ..." << endl;
	//	usleep(500000);		// Leave some time for FPGA to boot!
}

/*!
Writes to the FPGA boot source register, setting the boot source.
\param source - Long corresponding to the desired boot source.
 - 0x00000: Sector 0 of Flash Memory.
 - 0x00001: Sector 1 of Flash Memory.
 - 0x00002: Sector 2 of Flash Memory.
 - 0x00003: Sector 3 of Flash Memory.
 - 0x10000: ASRAM A.
*/
void 
XLM::SetFPGABoot(long source)
{
  //	vme[2] = source;
  //	long check = vme[2];
  //	if (check != source) {
  //		cout << "Failed to set boot source in " << getName() << " (slot " << getSlot() << " )" << endl;
  //		exit(1);
  //	}
}

/*!
Checks address 0x00000 of the FPGA Bus for a "signature" word written during configuration.
\param config - The signature word developed in the FPGA net file, configured by the user.
*/
void
XLM::CheckConfiguration(long config)
{
  //	string name = getName();
  //	config = long(Param_getVal(name.c_str(), "configuration"));
	//	AccessBus(0x10001);
	//	long check = fpga[0];
		long check = 0;
	//	ReleaseBus();
		if (check != config) {
		  //			cout << getName() << " XLM72 in slot " << getSlot() << " has wrong FPGA configuration" << endl;
	}
	else {
	  //		cout << getName() << " XLM72 in slot " << getSlot() << " successfully configured FPGA from:" << endl;
		cout << getFileName() << endl;
	}
}

/*!
You must do AcessBus(0x10001) before calling this routine.
 reads the memory locaion indicated.  You should call ReleaseBus() after you are finished reading this memory.
\param address - Integer corresponding to the address location to be read.  Address location is 
equal to four times the integer, since the lowest two address bits are always zero.
*/
long
XLM::ReadSRAMA(int address)
{

  long value;
  //  AccessBus(0x10001);  (moved external for efficiency)
  
  //  value = srama[address];
  //  ReleaseBus();

  return 0;
}
/*! 
Bus Ownership Register
\param bus - Character A,B, or X
Returns:
- 0: Bus is free
- 1: Bus controlled by VMEBus
- 2: Bus Controlled by FPGA
- 3: Bus Contolled by DSP
- 4: Invalid bus selection
*/

int
XLM::GetOwner(char bus)
{
  int value;
  if((bus=='A')||(bus=='a'))
    value=ownreg[0];
  else if((bus=='B')||(bus=='b'))
    value=ownreg[1];
  else if((bus=='X')||(bus=='x'))
    value=ownreg[2];
  else
    value = 4;

  return value;
}

/*! 
Reads ASRAM B.  Same Function as in SRAMA.  
\param address - Same usage as in ReadSRAMA.
*/
long
XLM::ReadSRAMB(int address)
{

  long value;
  //  AccessBus(0x10002);
  
  //  value = sramb[address];
  //  ReleaseBus();

  return 0;
}

/*!
You must do AcessBus(0x10001) before calling this routine.
 You should call ReleaseBus() after you are finished reading this memory.
Reads the FPGA Bus (X).  Same usage and functionality for ReadSRAMA and ReadSRAMB.
\param address - Same usage as ReadSRAMA and ReadSRAMB.
*/
long
XLM::ReadFPGA(int address)
{

  long value;
  printf("ReadFPGA %x\n",address);
  //  AccessBus(0x10000);
  
  //  value = fpga[address];
  //  ReleaseBus();

  return 0;
}
 
/*!
You must do AcessBus(0x10001) before calling this routine.
  You should call ReleaseBus() after you are finished reading this memory.
Accesses the FPGA bus, and writes to the indicated register.  Releases the bus upon completion.
\param address - FPGA Bus address location to be written to.  Note that the actual address is
four times the parameter, since the lowest two bits are always zero.
\param value - Value to be written to the address location.
*/
void
XLM::WriteFPGA(int address,long value)
{
  printf("WriteFPGA addr %x value %lx\n",address,value);
}

/*!
You must do AcessBus(0x10001) before calling this routine.
  You should call ReleaseBus() after you are finished reading this memory.Accesses the ASRAM A, and writes to the indicated register.  Releases the bus upon completion.
\param address - ASRAM A address location to be written to.  Note that the actual address is
four times the parameter, since the lowest two bits are always zero.
\param value - Value to be written to the address location.
*/
void
XLM::WriteSRAMA(int address,long value)
{
  //  AccessBus(0x10001);
  //  srama[address]=value;
  //  ReleaseBus();
}

/*!
You must do AcessBus(0x10001) before calling this routine.
  You should call ReleaseBus() after you are finished reading this memory.Accesses the ASRAM B, and writes to the indicated register.  Releases the bus upon completion.
\param address - ASRAM B address location to be written to.  Note that the actual address is
four times the parameter, since the lowest two bits are always zero.
\param value - Value to be written to the address location.
*/
void
XLM::WriteSRAMB(int address,long value)
{
  //  AccessBus(0x10002);
  //  sramb[address]=value;
  //  ReleaseBus();
}
/***** list operations *****/
void
XLM::initMyList()
{
  printf("initList\n");
  CVMUSBReadoutList VMEList;
  numReads = 0;
  numWrites = 0;
  numDelays = 0;
}

void
XLM::addWrite32(int address, long value)
{
  //  printf("addWrite32 addr %x value %lx\n",XLM_Base+address,value);
  VMEList.addWrite32(XLM_Base+address, 0x09, value);
  numWrites++;
}

void
XLM::addRead32(int address)
{
  //  printf("addRead32 addr %x\n",address);
  VMEList.addRead32(XLM_Base+address, 0x09);
  numReads++;
}

void
XLM::addDelay(long value)
{
  //  printf("addDelay %lx\n",value);
  VMEList.addDelay(value);
  numDelays++;
}

int
XLM::executeList(long* response, unsigned int length, int* bytesRead)
{
  size_t value;
  int i,retval;
  long* resp;
  *response = 0x76543210;
  retval = Globals::pUSBController->executeList(VMEList, response, length, &value);
  *bytesRead =int(value);
  printf("execute list resp length %d\n",length);
  printf("retval %d, bytesRead %d\n",retval,value);
  printf("response buf at %lx\n",response);
  for (i=0;i<32;i++) {
    resp = response+long(i*4);
    printf("%d %lx\n",i,*resp);
  }
  printf("%d Write32s\n",numWrites);
  printf("%d Read32s\n",numReads);
  printf("%d Delays\n",numDelays);
}

/***************************************/

unsigned long
XLM::CheckFPGAMail()
{
	int attempts = 100;
	unsigned long check = 0;
	do {
		check = mail[0]&0xFFFF;
		attempts--;
	} while (check == 0 && attempts >= 0);
	if (check == 0) {
	  //		cout << "VME failed to detect mail from FPGA in " << getName() << " (slot " << getSlot() << " )" << endl;
	}
	ClearFPGAMail();
	return check;
}

void
XLM::ClearFPGAMail()
{
	clearmail[0] = 0x824;
}

// DSP Functions

void
XLM::ResetDSP()
{
	vme[1] = 2;
	vme[1] = 0;
	usleep(500000);		// Leave some time for DSP to reset!
}

void
XLM::BootDSP()
{
	ReadHPC();
	WriteHPC(hpc|0x00020002);
	cout << "Booting DSP ..." << endl;
	usleep(100000);		// Leave some time for DSP to boot!
}
	
long
XLM::ReadHPC()
{
	unsigned long low, high;
	high = dsp[0x100];
	// do it twice because of HPI bug
	high = dsp[0x100];
	low = dsp[0x101];
	hpc = low | (high<<16);
	return hpc;
}

void
XLM::WriteHPC(long value)
{
	unsigned long low, high;
	hpc = value;
	high = hpc>>16;
	low = hpc&0xFFFF;
	dsp[0] = high;
	dsp[1] = low;
}

long
XLM::ReadHPA()
{
	unsigned long low, high;
	high = dsp[0x102];
	// do it twice because of HPI bug
	high = dsp[0x102];
	low = dsp[0x103];
	hpa = low | (high<<16);
	return hpa;
}

void
XLM::WriteHPA(long value)
{
	unsigned long low, high;
	hpa = value;
	high = hpa>>16;
	low = hpa&0xFFFF;
	dsp[2] = high;
	dsp[3] = low;
}

long
XLM::ReadHPD()
{
	unsigned long low, high;
	high = dsp[0x106];
	// do it twice because of HPI bug
	high = dsp[0x106];
	low = dsp[0x107];
	hpd = low | (high<<16);
	return hpd;
}

void
XLM::WriteHPD(long value)
{
	unsigned long low, high;
	hpd = value;
	high = hpd>>16;
	low = hpd&0xFFFF;
	dsp[6] = high;
	dsp[7] = low;
}

unsigned long
XLM::CheckDSPMail()
{
	int attempts = 10000;
	unsigned long check = 0;
	do {
		check = (mail[0]&0xFFFF0000)>>16;
		attempts--;
	} while (check == 0 && attempts >= 0);
	if (check == 0) {
	  //		cout << "VME failed to detect mail from DSP in " << getName() << " (slot " << getSlot() << " )" << endl;
//	} else {
//		cout << "Mail obtained from DSP after " << 10000-attempts << " VME cycles" << endl;
	}
	ClearDSPMail();
	return check;
}

void
XLM::WaitForDSPFlag()
{
	int attempts =  10000;
	unsigned long check = 0x04000000;
	do {
		check = vme[0]&0x04000000;
		attempts--;
	} while (check == 0x04000000 && attempts >= 0);
	if (check == 0x04000000) {
	  //		cout << "DSP did not clear its flag in " << getName() << " (slot " << getSlot() << " )" << endl;
	} else {
		cout << "DSP flag cleared after " << 10000-attempts << " VME cycles" << endl;
	}
}

void
XLM::ClearDSPMail()
{
	clearmail[0] = 0x1048;
}

void
XLM::SendDSPMail(int mail)
{
	dspmail[0] = mail;
}

void
XLM::LoadDSP(char dspfile[80])
{
	FILE *file;
	short data, nbytes, bytes[25], length;
	char str[80];
	//	string name = getName();
	//fileName = Param_getStrVal(name.c_str(), "DSPCode");
	file = fopen(dspfile, "r");
	if (!file) {
		cout << "Failed to open DSP code file " << fileName << endl;
		exit(1);
	}
	// skip first data
	fgets(str, 80, file);
	// reset HPI address to 0
	hpa = 0;
	// Load code in DSP through HPI
	while (!feof(file)) {
		fgets(str, 80, file);
		length = strlen(str);
		if (length == 9) {
			str[0] = 48;
			str[1] = 120;
			sscanf(str, "%lx,", &hpa);
//			cout << "Found jump to address " << hex << hpa << endl;
		} else {
			nbytes = (strlen(str)-1)/3;
			sscanf(str, "%hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx %hx ",
			&bytes[0], &bytes[1], &bytes[2], &bytes[3], &bytes[4], &bytes[5], &bytes[6], &bytes[7], &bytes[8], &bytes[9],
			&bytes[10], &bytes[11], &bytes[12], &bytes[13], &bytes[14], &bytes[15], &bytes[16], &bytes[17], &bytes[18], &bytes[19],
			&bytes[20], &bytes[21], &bytes[22], &bytes[23]);
			for (short i=0; i<nbytes; i+=4) {
				// First byte
				hpd = bytes[i]<<24;
				// second byte
				hpd |= bytes[i+1]<<16;
				// third byte
				hpd |= bytes[i+2]<<8;
				// fourth byte
				hpd |= bytes[i+3];
				WriteHPA(hpa);
				WriteHPD(hpd);
				hpa += 4;
			}
		}
	}
	fclose(file);
	//	cout << getName() << " XLM72 in slot " << getSlot() << " successfully loaded DSP code from:" << endl;
	cout << fileName << endl;
}

